@@ -15,23 +15,23 @@
 """Naively converts tox.ini files into noxfile.py files."""
 
 import argparse
-import io
 import pkgutil
+from typing import Any, Iterator
 
 import jinja2
 import tox.config
 
 _TEMPLATE = jinja2.Template(
-    pkgutil.get_data(__name__, "tox_to_nox.jinja2").decode("utf-8"),
+    pkgutil.get_data(__name__, "tox_to_nox.jinja2").decode("utf-8"),  # type: ignore[union-attr]
     extensions=["jinja2.ext.do"],
 )
 
 
-def wrapjoin(seq):
-    return ", ".join(["'{}'".format(item) for item in seq])
+def wrapjoin(seq: Iterator[Any]) -> str:
+    return ", ".join([f"'{item}'" for item in seq])
 
 
-def main():
+def main() -> None:
     parser = argparse.ArgumentParser(description="Converts toxfiles to noxfiles.")
     parser.add_argument("--output", default="noxfile.py")
 
@@ -40,5 +40,5 @@ def main():
     config = tox.config.parseconfig([])
     output = _TEMPLATE.render(config=config, wrapjoin=wrapjoin)
 
-    with io.open(args.output, "w") as outfile:
+    with open(args.output, "w") as outfile:
         outfile.write(output)
