@@ -18,4 +18,4 @@ import nox
 @nox.session(py=False)
 @nox.parametrize("cheese", ["cheddar", "jack", "brie"])
 def snack(unused_session, cheese):
-    print("Noms, {} so good!".format(cheese))
+    print(f"Noms, {cheese} so good!")
