@@ -12,12 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import contextlib
 import os
 import sys
+from pathlib import Path
 from unittest import mock
 
-import contexter
-import pkg_resources
 import pytest
 
 import nox
@@ -26,9 +26,14 @@ import nox._options
 import nox.registry
 import nox.sessions
 
+try:
+    import importlib.metadata as metadata
+except ImportError:
+    import importlib_metadata as metadata
+
 
 RESOURCES = os.path.join(os.path.dirname(__file__), "resources")
-VERSION = pkg_resources.get_distribution("nox").version
+VERSION = metadata.version("nox")
 
 
 # This is needed because CI systems will mess up these tests due to the
@@ -52,6 +57,7 @@ def test_main_no_args(monkeypatch):
         config = execute.call_args[1]["global_config"]
         assert config.noxfile == "noxfile.py"
         assert config.sessions is None
+        assert not config.no_venv
         assert not config.reuse_existing_virtualenvs
         assert not config.stop_on_first_error
         assert config.posargs == []
@@ -67,6 +73,11 @@ def test_main_long_form_args():
         "--sessions",
         "1",
         "2",
+        "--default-venv-backend",
+        "venv",
+        "--force-venv-backend",
+        "none",
+        "--no-venv",
         "--reuse-existing-virtualenvs",
         "--stop-on-first-error",
     ]
@@ -84,14 +95,72 @@ def test_main_long_form_args():
         assert config.noxfile == "noxfile.py"
         assert config.envdir.endswith(".other")
         assert config.sessions == ["1", "2"]
+        assert config.default_venv_backend == "venv"
+        assert config.force_venv_backend == "none"
+        assert config.no_venv is True
         assert config.reuse_existing_virtualenvs is True
         assert config.stop_on_first_error is True
         assert config.posargs == []
 
 
+def test_main_no_venv(monkeypatch, capsys):
+    # Check that --no-venv overrides force_venv_backend
+    monkeypatch.setattr(
+        sys,
+        "argv",
+        [
+            "nox",
+            "--noxfile",
+            os.path.join(RESOURCES, "noxfile_pythons.py"),
+            "--no-venv",
+            "-s",
+            "snack(cheese='cheddar')",
+        ],
+    )
+
+    with mock.patch("sys.exit") as sys_exit:
+        nox.__main__.main()
+        stdout, stderr = capsys.readouterr()
+        assert stdout == "Noms, cheddar so good!\n"
+        assert (
+            "Session snack is set to run with venv_backend='none', IGNORING its python"
+            in stderr
+        )
+        assert "Session snack(cheese='cheddar') was successful." in stderr
+        sys_exit.assert_called_once_with(0)
+
+
+def test_main_no_venv_error():
+    # Check that --no-venv can not be set together with a non-none --force-venv-backend
+    sys.argv = [
+        sys.executable,
+        "--noxfile",
+        "noxfile.py",
+        "--force-venv-backend",
+        "conda",
+        "--no-venv",
+    ]
+    with pytest.raises(ValueError, match="You can not use"):
+        nox.__main__.main()
+
+
 def test_main_short_form_args(monkeypatch):
     monkeypatch.setattr(
-        sys, "argv", [sys.executable, "-f", "noxfile.py", "-s", "1", "2", "-r"]
+        sys,
+        "argv",
+        [
+            sys.executable,
+            "-f",
+            "noxfile.py",
+            "-s",
+            "1",
+            "2",
+            "-db",
+            "venv",
+            "-fb",
+            "conda",
+            "-r",
+        ],
     )
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 0
@@ -106,6 +175,8 @@ def test_main_short_form_args(monkeypatch):
         config = execute.call_args[1]["global_config"]
         assert config.noxfile == "noxfile.py"
         assert config.sessions == ["1", "2"]
+        assert config.default_venv_backend == "venv"
+        assert config.force_venv_backend == "conda"
         assert config.reuse_existing_virtualenvs is True
 
 
@@ -166,20 +237,27 @@ def test_main_session_from_nox_env_var(monkeypatch, env, sessions):
             assert session in config.sessions
 
 
-def test_main_positional_args(monkeypatch):
+def test_main_positional_args(capsys, monkeypatch):
+    fake_exit = mock.Mock(side_effect=ValueError("asdf!"))
+
     monkeypatch.setattr(sys, "argv", [sys.executable, "1", "2", "3"])
-    with mock.patch("nox.workflow.execute") as execute:
-        execute.return_value = 0
+    with mock.patch.object(sys, "exit", fake_exit), pytest.raises(
+        ValueError, match="asdf!"
+    ):
+        nox.__main__.main()
+    _, stderr = capsys.readouterr()
+    assert "Unknown argument(s) '1 2 3'" in stderr
+    fake_exit.assert_called_once_with(2)
 
-        # Call the main function.
-        with mock.patch.object(sys, "exit") as exit:
-            nox.__main__.main()
-            exit.assert_called_once_with(0)
-        assert execute.called
-
-        # Verify that the positional args are listed in the config.
-        config = execute.call_args[1]["global_config"]
-        assert config.posargs == ["1", "2", "3"]
+    fake_exit.reset_mock()
+    monkeypatch.setattr(sys, "argv", [sys.executable, "1", "2", "3", "--"])
+    with mock.patch.object(sys, "exit", fake_exit), pytest.raises(
+        ValueError, match="asdf!"
+    ):
+        nox.__main__.main()
+    _, stderr = capsys.readouterr()
+    assert "Unknown argument(s) '1 2 3'" in stderr
+    fake_exit.assert_called_once_with(2)
 
 
 def test_main_positional_with_double_hyphen(monkeypatch):
@@ -219,7 +297,7 @@ def test_main_positional_flag_like_with_double_hyphen(monkeypatch):
 def test_main_version(capsys, monkeypatch):
     monkeypatch.setattr(sys, "argv", [sys.executable, "--version"])
 
-    with contexter.ExitStack() as stack:
+    with contextlib.ExitStack() as stack:
         execute = stack.enter_context(mock.patch("nox.workflow.execute"))
         exit_mock = stack.enter_context(mock.patch("sys.exit"))
         nox.__main__.main()
@@ -232,7 +310,7 @@ def test_main_version(capsys, monkeypatch):
 def test_main_help(capsys, monkeypatch):
     monkeypatch.setattr(sys, "argv", [sys.executable, "--help"])
 
-    with contexter.ExitStack() as stack:
+    with contextlib.ExitStack() as stack:
         execute = stack.enter_context(mock.patch("nox.workflow.execute"))
         exit_mock = stack.enter_context(mock.patch("sys.exit"))
         nox.__main__.main()
@@ -293,6 +371,78 @@ def test_main_session_with_names(capsys, monkeypatch):
         sys_exit.assert_called_once_with(0)
 
 
+@pytest.fixture
+def run_nox(capsys, monkeypatch):
+    def _run_nox(*args):
+        monkeypatch.setattr(sys, "argv", ["nox", *args])
+
+        with mock.patch("sys.exit") as sys_exit:
+            nox.__main__.main()
+            stdout, stderr = capsys.readouterr()
+            returncode = sys_exit.call_args[0][0]
+
+        return returncode, stdout, stderr
+
+    return _run_nox
+
+
+@pytest.mark.parametrize(
+    ("normalized_name", "session"),
+    [
+        ("test(arg='Jane')", "test(arg='Jane')"),
+        ("test(arg='Jane')", 'test(arg="Jane")'),
+        ("test(arg='Jane')", 'test(arg = "Jane")'),
+        ("test(arg='Jane')", 'test ( arg = "Jane" )'),
+        ('test(arg="Joe\'s")', 'test(arg="Joe\'s")'),
+        ('test(arg="Joe\'s")', "test(arg='Joe\\'s')"),
+        ("test(arg='\"hello world\"')", "test(arg='\"hello world\"')"),
+        ("test(arg='\"hello world\"')", 'test(arg="\\"hello world\\"")'),
+        ("test(arg=[42])", "test(arg=[42])"),
+        ("test(arg=[42])", "test(arg=[42,])"),
+        ("test(arg=[42])", "test(arg=[ 42 ])"),
+        ("test(arg=[42])", "test(arg=[0x2a])"),
+        (
+            "test(arg=datetime.datetime(1980, 1, 1, 0, 0))",
+            "test(arg=datetime.datetime(1980, 1, 1, 0, 0))",
+        ),
+        (
+            "test(arg=datetime.datetime(1980, 1, 1, 0, 0))",
+            "test(arg=datetime.datetime(1980,1,1,0,0))",
+        ),
+        (
+            "test(arg=datetime.datetime(1980, 1, 1, 0, 0))",
+            "test(arg=datetime.datetime(1980, 1, 1, 0, 0x0))",
+        ),
+    ],
+)
+def test_main_with_normalized_session_names(run_nox, normalized_name, session):
+    noxfile = os.path.join(RESOURCES, "noxfile_normalization.py")
+    returncode, _, stderr = run_nox(f"--noxfile={noxfile}", f"--session={session}")
+    assert returncode == 0
+    assert normalized_name in stderr
+
+
+@pytest.mark.parametrize(
+    "session",
+    [
+        "syntax error",
+        "test(arg=Jane)",
+        "test(arg='Jane ')",
+        "_test(arg='Jane')",
+        "test(arg=42)",
+        "test(arg=[42.0])",
+        "test(arg=[43])",
+        "test(arg=<user_nox_module.Foo object at 0x123456789abc>)",
+        "test(arg=datetime.datetime(1980, 1, 1))",
+    ],
+)
+def test_main_with_bad_session_names(run_nox, session):
+    noxfile = os.path.join(RESOURCES, "noxfile_normalization.py")
+    returncode, _, stderr = run_nox(f"--noxfile={noxfile}", f"--session={session}")
+    assert returncode != 0
+    assert session in stderr
+
+
 def test_main_noxfile_options(monkeypatch):
     monkeypatch.setattr(
         sys,
@@ -368,12 +518,95 @@ def test_main_noxfile_options_sessions(monkeypatch):
         assert config.sessions == ["test"]
 
 
+@pytest.fixture
+def generate_noxfile_options_pythons(tmp_path):
+    """Generate noxfile.py with test and launch_rocket sessions.
+
+    The sessions are defined for both the default and alternate Python versions.
+    The ``default_session`` and ``default_python`` parameters determine what
+    goes into ``nox.options.sessions`` and ``nox.options.pythons``, respectively.
+    """
+
+    def generate_noxfile(default_session, default_python, alternate_python):
+        path = Path(RESOURCES) / "noxfile_options_pythons.py"
+        text = path.read_text()
+        text = text.format(
+            default_session=default_session,
+            default_python=default_python,
+            alternate_python=alternate_python,
+        )
+        path = tmp_path / "noxfile.py"
+        path.write_text(text)
+        return str(path)
+
+    return generate_noxfile
+
+
+python_current_version = f"{sys.version_info.major}.{sys.version_info.minor}"
+python_next_version = f"{sys.version_info.major}.{sys.version_info.minor + 1}"
+
+
+def test_main_noxfile_options_with_pythons_override(
+    capsys, monkeypatch, generate_noxfile_options_pythons
+):
+    noxfile = generate_noxfile_options_pythons(
+        default_session="test",
+        default_python=python_next_version,
+        alternate_python=python_current_version,
+    )
+
+    monkeypatch.setattr(
+        sys, "argv", ["nox", "--noxfile", noxfile, "--python", python_current_version]
+    )
+
+    with mock.patch("sys.exit") as sys_exit:
+        nox.__main__.main()
+        _, stderr = capsys.readouterr()
+        sys_exit.assert_called_once_with(0)
+
+    for python_version in [python_current_version, python_next_version]:
+        for session in ["test", "launch_rocket"]:
+            line = f"Running session {session}-{python_version}"
+            if session == "test" and python_version == python_current_version:
+                assert line in stderr
+            else:
+                assert line not in stderr
+
+
+def test_main_noxfile_options_with_sessions_override(
+    capsys, monkeypatch, generate_noxfile_options_pythons
+):
+    noxfile = generate_noxfile_options_pythons(
+        default_session="test",
+        default_python=python_current_version,
+        alternate_python=python_next_version,
+    )
+
+    monkeypatch.setattr(
+        sys, "argv", ["nox", "--noxfile", noxfile, "--session", "launch_rocket"]
+    )
+
+    with mock.patch("sys.exit") as sys_exit:
+        nox.__main__.main()
+        _, stderr = capsys.readouterr()
+        sys_exit.assert_called_once_with(0)
+
+    for python_version in [python_current_version, python_next_version]:
+        for session in ["test", "launch_rocket"]:
+            line = f"Running session {session}-{python_version}"
+            if session == "launch_rocket" and python_version == python_current_version:
+                assert line in stderr
+            else:
+                assert line not in stderr
+
+
 @pytest.mark.parametrize(("isatty_value", "expected"), [(True, True), (False, False)])
 def test_main_color_from_isatty(monkeypatch, isatty_value, expected):
+    monkeypatch.delenv("FORCE_COLOR", raising=False)
     monkeypatch.setattr(sys, "argv", [sys.executable])
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 0
-        with mock.patch("sys.stdin.isatty") as isatty:
+        with mock.patch("sys.stdout.isatty") as isatty:
             isatty.return_value = isatty_value
 
             # Call the main function.
@@ -394,6 +627,7 @@ def test_main_color_from_isatty(monkeypatch, isatty_value, expected):
     ],
 )
 def test_main_color_options(monkeypatch, color_opt, expected):
+    monkeypatch.delenv("FORCE_COLOR", raising=False)
     monkeypatch.setattr(sys, "argv", [sys.executable, color_opt])
     with mock.patch("nox.workflow.execute") as execute:
         execute.return_value = 0
@@ -419,3 +653,21 @@ def test_main_color_conflict(capsys, monkeypatch):
     _, err = capsys.readouterr()
 
     assert "color" in err
+
+
+def test_main_force_python(monkeypatch):
+    monkeypatch.setattr(sys, "argv", ["nox", "--force-python=3.10"])
+    with mock.patch("nox.workflow.execute", return_value=0) as execute:
+        with mock.patch.object(sys, "exit"):
+            nox.__main__.main()
+        config = execute.call_args[1]["global_config"]
+    assert config.pythons == config.extra_pythons == ["3.10"]
+
+
+def test_main_reuse_existing_virtualenvs_no_install(monkeypatch):
+    monkeypatch.setattr(sys, "argv", ["nox", "-R"])
+    with mock.patch("nox.workflow.execute", return_value=0) as execute:
+        with mock.patch.object(sys, "exit"):
+            nox.__main__.main()
+        config = execute.call_args[1]["global_config"]
+    assert config.reuse_existing_virtualenvs and config.no_install
